/* GuiLaunchHandler.java
   Copyright (C) 2012 Red Hat, Inc.

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */

package net.sourceforge.jnlp;

import java.net.URL;
import net.sourceforge.swing.SwingUtils;

import net.sourceforge.jnlp.cache.ResourceTracker;
import net.sourceforge.jnlp.cache.UpdatePolicy;
import net.sourceforge.jnlp.runtime.ApplicationInstance;
import net.sourceforge.jnlp.util.BasicExceptionDialog;
import net.sourceforge.jnlp.util.logging.OutputController;

/**
 * A {@link LaunchHandler} that gives feedback to the user using GUI elements
 * including splash screens and exception dialogs.
 */
public class GuiLaunchHandler extends AbstractLaunchHandler {

    private volatile JNLPSplashScreen splashScreen = null;
    private final Object mutex = new Object();
    private UpdatePolicy policy = UpdatePolicy.ALWAYS;
    private volatile KeepAWTAliveThread keepAWTAliveThread = null;

    public GuiLaunchHandler(OutputController outputStream) {
        super(outputStream);
    }

    @Override
    public void launchCompleted(ApplicationInstance application) {
        synchronized (mutex) {
            if (keepAWTAliveThread != null) {
                keepAWTAliveThread.stopThread();
                keepAWTAliveThread = null;
            }
        }
    }

    @Override
    public void launchError(final LaunchException exception) {
        BasicExceptionDialog.willBeShown();
        SwingUtils.invokeLater(new Runnable() {
            @Override
            public void run() {
                closeSplashScreen();
                BasicExceptionDialog.show(exception);
            }
        });
        printMessage(exception);
    }

    private void closeSplashScreen() {
        synchronized (mutex) {
            if (splashScreen != null) {
                if (splashScreen.isSplashScreenValid()) {
                    splashScreen.setVisible(false);
                    splashScreen.stopAnimation();
                }
                splashScreen.dispose();
            }
        }
    }

    @Override
    public void launchStarting(ApplicationInstance application) {
        SwingUtils.invokeLater(new Runnable() {
            @Override
            public void run() {
                closeSplashScreen();
            }
        });
    }

    @Override
    @SuppressWarnings("empty-statement")
    public void launchInitialized(final JNLPFile file) {

        int preferredWidth = 500;
        int preferredHeight = 400;

        final URL splashImageURL = file.getInformation().getIconLocation(
                IconDesc.SPLASH, preferredWidth, preferredHeight);

        final ResourceTracker resourceTracker = new ResourceTracker(true);
        if (splashImageURL != null) {
            resourceTracker.addResource(splashImageURL, file.getFileVersion(), null, policy);
        }
        synchronized (mutex) {
            SwingUtils.invokeAndWait(new Runnable() {

                @Override
                public void run() {
                    splashScreen = new JNLPSplashScreen(resourceTracker, file);
                    splashScreen.setSplashImageURL(splashImageURL);

                    keepAWTAliveThread = new KeepAWTAliveThread();
                    keepAWTAliveThread.start();
                }
            });
        }

        SwingUtils.invokeLater(new Runnable() {

            @Override
            public void run() {
                if (splashScreen != null) {
                    synchronized (mutex) {
                        if (splashScreen.isSplashScreenValid()) {
                            splashScreen.setVisible(true);
                        }
                    }
                }
            }
        });
    }

    @Override
    public boolean launchWarning(LaunchException warning) {
        printMessage(warning);
        return true;
    }

    @Override
    public boolean validationError(LaunchException error) {
        closeSplashScreen();
        printMessage(error);
        return true;
    }

}

/*
 * There may be a situation when the application launch takes a long time without UI activity.
 * In this case, the AWTAutoShutdown thread will stop the EDT thread and this thread will be
 * recreated at the next UI activity. When recreating EDT, the JNLP class loader
 * (earlier set as context class loader of original EDT) will be lost.
 * The purpose of this thread is to keep AWT from re-creating EDT.  The interval (500ms) is
 * synchronized (should be less) with safety timeout (1000ms) in the AWTAutoShutdown thread.
 */
class KeepAWTAliveThread extends Thread {
    private static final int INTERVAL = 500;
    private volatile boolean stopped = false;

    KeepAWTAliveThread() {
        super("KeepAWTAlive");
    }

    @Override
    public void run() {
        while (!stopped) {
            synchronized (KeepAWTAliveThread.this) {
                try {
                    KeepAWTAliveThread.this.wait(INTERVAL);
                } catch (InterruptedException ignored) {
                }
            }
            SwingUtils.invokeLater(new Runnable() {
                @Override
                public void run() {
                }
            });
        }
    }

    void stopThread() {
        stopped = true;
    }
}
